@@ -597,7 +597,7 @@ _gnutls_openpgp_crt_verify_peers(gnutls_session_t session,
  * or use gnutls_certificate_verify_peers3().
  *
  * Returns: a negative error code on error and %GNUTLS_E_SUCCESS (0)
- * when the peer's certificate was successfully parsed, whether or not
+ * when the peer's certificate was successfully parsed, irrespective of whether
  * it was verified.
  **/
 int
@@ -630,8 +630,8 @@ gnutls_certificate_verify_peers2(gnutls_session_t session,
  * In order to verify the purpose of the end-certificate (by checking the extended
  * key usage), use gnutls_certificate_verify_peers().
  *
- * Returns: a negative error code on error and %GNUTLS_E_SUCCESS (0) 
- * when the peer's certificate was successfully parsed, whether or not
+ * Returns: a negative error code on error and %GNUTLS_E_SUCCESS (0)
+ * when the peer's certificate was successfully parsed, irrespective of whether
  * it was verified.
  *
  * Since: 3.1.4
@@ -677,7 +677,7 @@ gnutls_typed_vdata_st data;
  * or be marked for any purpose, otherwise verification will fail with %GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE status.
  *
  * Returns: a negative error code on error and %GNUTLS_E_SUCCESS (0)
- * when the peer's certificate was successfully parsed, whether or not
+ * when the peer's certificate was successfully parsed, irrespective of whether
  * it was verified.
  *
  * Since: 3.3.0
@@ -828,6 +828,9 @@ int _gnutls_check_key_cert_match(gnutls_certificate_credentials_t res)
 	gnutls_datum_t sig = {NULL, 0};
 	int pk, pk2, ret;
 
+	if (res->flags & GNUTLS_CERTIFICATE_SKIP_KEY_CERT_MATCH)
+		return 0;
+
 	pk =
 	    gnutls_pubkey_get_pk_algorithm(res->certs[res->ncerts - 1].
 					   cert_list[0].pubkey, NULL);
