@@ -340,7 +340,7 @@ void gnutls_system_global_deinit(void)
  */
 int _gnutls_find_config_path(char *path, size_t max_size)
 {
-	const char *home_dir = getenv("HOME");
+	const char *home_dir = secure_getenv("HOME");
 
 	if (home_dir != NULL && home_dir[0] != 0) {
 		snprintf(path, max_size, "%s/" CONFIG_PATH, home_dir);
@@ -615,8 +615,6 @@ int _gnutls_ucs2_to_utf8(const void *data, size_t size,
 		size -= 2;
 	}
 
-	src_len = wcslen(data);
-
 	src = gnutls_malloc(size+2);
 	if (src == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
@@ -633,6 +631,8 @@ int _gnutls_ucs2_to_utf8(const void *data, size_t size,
 	src[size] = 0;
 	src[size+1] = 0;
 
+	src_len = wcslen(src);
+
 	ret =
 	    WideCharToMultiByte(CP_UTF8, flags,
 				(void *) src, src_len, NULL, 0,
