@@ -1,8 +1,9 @@
 /*
- * Copyright (C) 2004-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2004-2016 Free Software Foundation, Inc.
  * Copyright (C) 2013 Adam Sampson <ats@offog.org>
+ * Copyright (C) 2016 Red Hat, Inc.
  *
- * Author: Simon Josefsson
+ * Author: Simon Josefsson, Nikos Mavrogiannopoulos
  *
  * This file is part of GnuTLS.
  *
@@ -68,16 +69,43 @@ struct params_res {
 	int enable_session_ticket_server;
 	int enable_session_ticket_client;
 	int expect_resume;
+	int try_alpn;
 };
 
 pid_t child;
 
 struct params_res resume_tests[] = {
-	{"try to resume from db", 1, 0, 0, 1},
-	{"try to resume from session ticket", 0, 1, 1, 1},
-	{"try to resume from session ticket (server only)", 0, 1, 0, 0},
-	{"try to resume from session ticket (client only)", 0, 0, 1, 0},
-	{"try to resume from db and ticket", 1, 1, 1, 1},
+	{.desc = "try to resume from db",
+	 .enable_db = 1,
+	 .enable_session_ticket_server = 0,
+	 .enable_session_ticket_client = 0,
+	 .expect_resume = 1},
+	{.desc = "try to resume from db and check ALPN",
+	 .enable_db = 1,
+	 .enable_session_ticket_server = 0,
+	 .enable_session_ticket_client = 0,
+	 .try_alpn = 1,
+	 .expect_resume = 1},
+	{.desc = "try to resume from session ticket", 
+	 .enable_db = 0, 
+	 .enable_session_ticket_server = 1,
+	 .enable_session_ticket_client = 1,
+	 .expect_resume = 1},
+	{.desc = "try to resume from session ticket (server only)",
+	  .enable_db = 0,
+	  .enable_session_ticket_server = 1,
+	  .enable_session_ticket_client = 0,
+	  .expect_resume = 0},
+	{.desc = "try to resume from session ticket (client only)",
+	 .enable_db = 0,
+	 .enable_session_ticket_server = 0,
+	 .enable_session_ticket_client = 1,
+	 .expect_resume = 0},
+	{.desc = "try to resume from db and ticket",
+	 .enable_db = 1,
+	 .enable_session_ticket_server = 1,
+	 .enable_session_ticket_client = 1,
+	 .expect_resume = 1},
 	{NULL, -1}
 };
 
@@ -88,18 +116,100 @@ struct params_res resume_tests[] = {
 #define MAX_BUF 5*1024
 #define MSG "Hello TLS"
 
+#define HANDSHAKE_SESSION_ID_POS (2+32)
+
 static void tls_log_func(int level, const char *str)
 {
 	fprintf(stderr, "%s |<%d>| %s", child ? "server" : "client", level,
 		str);
 }
 
+static int hsk_hook_cb(gnutls_session_t session, unsigned int htype, unsigned post,
+		       unsigned int incoming, const gnutls_datum_t *_msg)
+{
+	unsigned size;
+	gnutls_datum msg = {_msg->data, _msg->size};
+
+	/* skip up to session ID */
+	if (msg.size <= HANDSHAKE_SESSION_ID_POS+6) {
+		fail("Cannot parse server hello\n");
+		return -1;
+	}
+
+	msg.data += HANDSHAKE_SESSION_ID_POS;
+	msg.size -= HANDSHAKE_SESSION_ID_POS;
+	size = msg.data[0];
+
+	if (msg.size <= size) {
+		fail("Cannot parse server hello 2\n");
+		return -1;
+	}
+
+	msg.data += size;
+	msg.size -= size;
+
+	if (memmem(msg.data, msg.size, "\x00\x17\x00\x00", 4) == 0) {
+		fail("Extended master secret extension was not found in resumed session hello\n");
+		exit(1);
+	}
+	return 0;
+}
+
+static void append_alpn(gnutls_session_t session, struct params_res *params, unsigned alpn_counter)
+{
+	gnutls_datum_t protocol;
+	int ret;
+	char str[64];
+
+	if (!params->try_alpn)
+		return;
+
+	snprintf(str, sizeof(str), "myproto-%d", alpn_counter);
+
+	protocol.data = (void*)str;
+	protocol.size = strlen(str);
+
+	ret = gnutls_alpn_set_protocols(session, &protocol, 1, 0);
+	if (ret < 0) {
+		gnutls_perror(ret);
+		exit(1);
+	}
+}
+
+static void verify_alpn(gnutls_session_t session, struct params_res *params, unsigned alpn_counter)
+{
+	int ret;
+	gnutls_datum_t selected;
+	char str[64];
+
+	if (!params->try_alpn)
+		return;
+
+	snprintf(str, sizeof(str), "myproto-%d", alpn_counter);
+
+	ret = gnutls_alpn_get_selected_protocol(session, &selected);
+	if (ret < 0) {
+		gnutls_perror(ret);
+		exit(1);
+	}
+
+	if (strlen(str) != selected.size || memcmp(str, selected.data, selected.size) != 0) {
+		fail("expected protocol %s, got %.*s\n", str, selected.size, selected.data);
+		exit(1);
+	}
+
+	if (debug)
+		success("ALPN got: %s\n", str);
+}
+
 static void client(int sds[], struct params_res *params)
 {
 	int ret, ii;
 	gnutls_session_t session;
 	char buffer[MAX_BUF + 1];
 	gnutls_anon_client_credentials_t anoncred;
+	unsigned int ext_master_secret = 0;
+
 	/* Need to enable anonymous KX specifically. */
 
 	/* variables used in session resuming
@@ -133,6 +243,7 @@ static void client(int sds[], struct params_res *params)
 						   "NONE:+VERS-TLS-ALL:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-DH:%NO_TICKETS",
 						   NULL);
 		}
+		append_alpn(session, params, t);
 
 		/* put the anonymous credentials to the current session
 		 */
@@ -144,6 +255,8 @@ static void client(int sds[], struct params_res *params)
 						session_data.size);
 		}
 
+		if (ext_master_secret)
+			gnutls_handshake_set_hook_function(session, GNUTLS_HANDSHAKE_SERVER_HELLO, GNUTLS_HOOK_PRE, hsk_hook_cb);
 		gnutls_transport_set_int(session, sd);
 
 		/* Perform the TLS handshake
@@ -163,13 +276,17 @@ static void client(int sds[], struct params_res *params)
 				    ("client: Handshake was completed\n");
 		}
 
-		if (t == 0) {	/* the first time we connect */
+		ext_master_secret = 0;
+		if (t == 0) {
+			ext_master_secret =  gnutls_session_ext_master_secret_status(session);
+
 			/* get the session data size */
 			ret =
 			    gnutls_session_get_data2(session,
 						     &session_data);
 			if (ret < 0)
 				fail("Getting resume data failed\n");
+
 		} else {	/* the second time we connect */
 
 			/* check if we actually resumed the previous session */
@@ -191,6 +308,8 @@ static void client(int sds[], struct params_res *params)
 			}
 		}
 
+		verify_alpn(session, params, t);
+
 		gnutls_record_send(session, MSG, strlen(MSG));
 
 		ret = gnutls_record_recv(session, buffer, MAX_BUF);
@@ -230,7 +349,6 @@ static void client(int sds[], struct params_res *params)
 #define DH_BITS 1024
 
 /* These are global */
-gnutls_anon_server_credentials_t anoncred;
 static gnutls_datum_t session_ticket_key = { NULL, 0 };
 
 static gnutls_session_t initialize_tls_session(struct params_res *params)
@@ -246,7 +364,6 @@ static gnutls_session_t initialize_tls_session(struct params_res *params)
 				   "NONE:+VERS-TLS-ALL:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-DH",
 				   NULL);
 
-	gnutls_credentials_set(session, GNUTLS_CRD_ANON, anoncred);
 
 	gnutls_dh_set_prime_bits(session, DH_BITS);
 
@@ -265,6 +382,7 @@ static gnutls_session_t initialize_tls_session(struct params_res *params)
 }
 
 static gnutls_dh_params_t dh_params;
+gnutls_anon_server_credentials_t anoncred;
 
 static int generate_dh_params(void)
 {
@@ -279,11 +397,6 @@ static int generate_dh_params(void)
 					     GNUTLS_X509_FMT_PEM);
 }
 
-int err, ret;
-char topbuf[512];
-gnutls_session_t session;
-char buffer[MAX_BUF + 1];
-int optval = 1;
 
 static void global_stop(void)
 {
@@ -300,6 +413,9 @@ static void global_stop(void)
 static void server(int sds[], struct params_res *params)
 {
 	size_t t;
+	int ret;
+	gnutls_session_t session;
+	char buffer[MAX_BUF + 1];
 
 	/* this must be called once in the program, it is mostly for the server.
 	 */
@@ -330,6 +446,9 @@ static void server(int sds[], struct params_res *params)
 
 		session = initialize_tls_session(params);
 
+		append_alpn(session, params, t);
+
+		gnutls_credentials_set(session, GNUTLS_CRD_ANON, anoncred);
 		gnutls_transport_set_int(session, sd);
 		gnutls_handshake_set_timeout(session, 20 * 1000);
 		do {
@@ -346,6 +465,8 @@ static void server(int sds[], struct params_res *params)
 		if (debug)
 			success("server: Handshake was completed\n");
 
+		verify_alpn(session, params, t);
+
 		/* see the Getting peer's information example */
 		/* print_info(session); */
 
@@ -391,7 +512,7 @@ static void server(int sds[], struct params_res *params)
 
 void doit(void)
 {
-	int i;
+	int i, err;
 
 	signal(SIGCHLD, SIG_IGN);
 	signal(SIGPIPE, SIG_IGN);
